package eu.jonahbauer.survey.controller.survey;

import eu.jonahbauer.survey.model.SurveyCreateForm;
import eu.jonahbauer.survey.persistence.entity.Survey;
import eu.jonahbauer.survey.persistence.entity.User;
import eu.jonahbauer.survey.persistence.repository.SurveyRepository;
import jakarta.validation.Valid;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.security.core.Authentication;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import java.util.HashSet;

@Controller
@Getter(AccessLevel.PRIVATE)
@RequiredArgsConstructor
public class SurveyCreateController {
    private final SurveyRepository surveyRepository;

    @GetMapping("/survey/create")
    public String get(Model model) {
        model.addAttribute("survey", new SurveyCreateForm());

        return "survey/create";
    }

    @PostMapping("/survey/create")
    public String post(
            @Valid @ModelAttribute("survey") SurveyCreateForm surveyForm,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes,
            Authentication authentication
    ) {
        validateSurveyForm(surveyForm, bindingResult);

        if (bindingResult.hasErrors()) {
            model.addAttribute("survey", surveyForm);
            return "survey/create";
        } else {
            Survey survey = surveyForm.toSurvey();
            survey.setOwner((User) authentication.getPrincipal());

            survey = getSurveyRepository().save(survey);
            redirectAttributes.addAttribute("id", survey.getId());
            return "redirect:/survey/{id}/evaluate";
        }
    }

    private void validateSurveyForm(SurveyCreateForm surveyForm, BindingResult bindingResult) {
        HashSet<String> questions = new HashSet<>();
        HashSet<String> answers = new HashSet<>();
        boolean uniqueQuestions = true;

        if (surveyForm.getQuestions() != null) {
            int qIndex = -1;
            for (SurveyCreateForm.QuestionForm question : surveyForm.getQuestions()) {
                qIndex++;

                uniqueQuestions &= questions.add(question.getText());

                boolean uniqueAnswers = true;

                if (question.getAnswers() != null) {
                    answers.clear();
                    for (String answer : question.getAnswers()) {
                        uniqueAnswers &= answers.add(answer);
                    }
                }

                if (!uniqueAnswers) {
                    bindingResult.addError(new FieldError(
                            "survey", "questions[" + qIndex + "]",
                            question, false,
                            new String[] {"survey.questions.answers.unique"}, null, null
                    ));
                }
            }
        }

        if (!uniqueQuestions) {
            bindingResult.addError(new ObjectError(
                    "survey",
                    new String[] {"survey.questions.unique"}, null, null
            ));
        }
    }
}
