package eu.jonahbauer.survey.controller.survey;

import eu.jonahbauer.survey.persistence.entity.Answer;
import eu.jonahbauer.survey.persistence.entity.Question;
import eu.jonahbauer.survey.persistence.entity.Submission;
import eu.jonahbauer.survey.persistence.entity.Survey;
import eu.jonahbauer.survey.persistence.repository.SubmissionRepository;
import eu.jonahbauer.survey.persistence.repository.SurveyRepository;
import eu.jonahbauer.survey.model.SurveyParticipationForm;
import lombok.AccessLevel;
import lombok.Getter;
import lombok.RequiredArgsConstructor;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.validation.FieldError;
import org.springframework.validation.ObjectError;
import org.springframework.web.bind.annotation.*;

import java.util.*;
import java.util.stream.Collectors;

@Controller
@Getter(AccessLevel.PRIVATE)
@RequiredArgsConstructor
public class SurveyParticipationController {
    private final SurveyRepository surveyRepository;
    private final SubmissionRepository submissionRepository;

    @GetMapping("/survey/{id}/participate")
    public String get(@PathVariable int id, Model model) {
        Survey survey = getSurveyRepository().findDetailedById(id);
        model.addAttribute("survey", survey);
        model.addAttribute("form", new SurveyParticipationForm());
        return "survey/participate";
    }

    @PostMapping("/survey/{id}/participate")
    public String post(
            @PathVariable int id,
            @ModelAttribute("form") SurveyParticipationForm form,
            BindingResult bindingResult,
            Model model
    ) {
        Survey survey = getSurveyRepository().findDetailedById(id);

        validateParticipation(survey, form, bindingResult);

        if (bindingResult.hasErrors()) {
            model.addAttribute("survey", survey);
            return "survey/participate";
        } else {
            persistParticipation(survey, form);

            return "redirect:/survey/{id}/evaluate";
        }
    }

    private void validateParticipation(Survey survey, SurveyParticipationForm form, BindingResult bindingResult) {
        // validate all question ids are existent
        Set<Integer> questionIds = survey.getQuestions().stream().map(Question::getId).collect(Collectors.toSet());
        if (!questionIds.containsAll(form.getSelectedAnswers().keySet())) {
            bindingResult.addError(new ObjectError(
                    "form",
                    new String[] {"participation.survey.unknown_question"}, null, null
            ));
        }

        for (Question question : survey.getQuestions()) {
            Set<Integer> answerIds = question.getAnswers().stream().map(Answer::getId).collect(Collectors.toSet());
            Set<Integer> selectedAnswers = form.getSelectedAnswers().getOrDefault(question.getId(), Collections.emptySet());
            if (selectedAnswers == null) selectedAnswers = Collections.emptySet();

            // validate all answer ids are existent
            if (!answerIds.containsAll(selectedAnswers)) {
                bindingResult.addError(new FieldError(
                        "form", "selectedAnswers[" + question.getId() + "]",
                        null, false,
                        new String[] {"participation.question.unknown_answer"}, null, null
                ));
            } else {
                // validate min answer count
                if (selectedAnswers.size() < question.getMinAnswers()) {
                    bindingResult.addError(new FieldError(
                            "form", "selectedAnswers[" + question.getId() + "]",
                            null, false,
                            new String[] {"participation.question.not_enough_answers"}, new Object[] { question.getMinAnswers() }, null
                    ));
                }

                // validate max answer count
                if (selectedAnswers.size() > question.getMaxAnswers()) {
                    bindingResult.addError(new FieldError(
                            "form", "selectedAnswers[" + question.getId() + "]",
                            null, false,
                            new String[] {"participation.question.too_many_answers"}, new Object[] { question.getMaxAnswers() }, null
                    ));
                }
            }
        }
    }

    private void persistParticipation(Survey survey, SurveyParticipationForm form) {
        Submission submission = new Submission();
        submission.setSurvey(survey);
        submission.setAnswers(new HashSet<>());

        Map<Integer, Answer> answersById = survey.getQuestions().stream()
                .flatMap(q -> q.getAnswers().stream())
                .collect(Collectors.toMap(Answer::getId, a -> a));


        form.getSelectedAnswers().values().stream()
            .filter(Objects::nonNull)
            .flatMap(Collection::stream)
            .map(answersById::get)
            .forEach(submission.getAnswers()::add);

        getSubmissionRepository().save(submission);
    }
}
