package eu.jonahbauer.survey.model;

import eu.jonahbauer.survey.persistence.entity.Answer;
import eu.jonahbauer.survey.persistence.entity.Question;
import eu.jonahbauer.survey.persistence.entity.Survey;
import jakarta.validation.Valid;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import jakarta.validation.constraints.Size;
import lombok.Data;
import org.hibernate.validator.constraints.Length;

import java.util.List;
import java.util.TreeSet;

@Data
public final class SurveyCreateForm {
    public static final QuestionForm EMPTY_QUESTION = new QuestionForm();

    @NotBlank(message = "{survey.title.not_blank}")
    @Length(min = 8, max = 255, message = "{survey.title.length}")
    private String title;

    @NotBlank(message = "{survey.summary.not_blank}")
    @Length(min = 8, max = 255, message = "{survey.summary.length}")
    private String summary;

    @Size(min = 1, max = 20, message = "{survey.questions.size}")
    @NotNull(message = "{survey.questions.not_null}")
    @Valid
    private QuestionForm[] questions;

    public QuestionForm[] getQuestions() {
        return questions;
    }

    public SurveyCreateForm() {
        questions = new QuestionForm[] {new QuestionForm()};
    }

    public Survey toSurvey() {
        Survey survey = new Survey();
        survey.setTitle(title);
        survey.setSummary(summary);
        survey.setQuestions(new TreeSet<>());

        for (int i = 0; i < questions.length; i++) {
            Question question = questions[i].toQuestion();
            question.setNumber(i + 1);
            question.setSurvey(survey);
            survey.getQuestions().add(question);
        }

        return survey;
    }

    @Data
    public final static class QuestionForm {
        @NotNull(message = "{survey.questions.text.not_null}")
        @Length(min = 1, max = 255, message = "{survey.questions.text.length}")
        private String text;

        @Min(value = 0)
        private int minAnswers = 0;

        @Min(1)
        private int maxAnswers = 1;

        @NotNull(message = "{survey.questions.answers.not_null}")
        @Size(min = 1, max = 20, message = "{survey.questions.answers.size}")
        private List<@NotBlank(message = "{survey.questions.answers.not_blank}") String> answers;

        public Question toQuestion() {
            Question question = new Question();
            question.setText(text);

            int min = Math.min(minAnswers, maxAnswers);
            int max = Math.max(minAnswers, maxAnswers);
            min = clamp(0, answers.size(), min);
            max = clamp(1, answers.size(), max);

            question.setMinAnswers(min);
            question.setMaxAnswers(max);
            question.setAnswers(new TreeSet<>());

            for (int i = 0; i < answers.size(); i++) {
                Answer answer = new Answer();
                answer.setText(answers.get(i));
                answer.setNumber(i + 1);
                answer.setQuestion(question);
                question.getAnswers().add(answer);
            }

            return question;
        }

        private int clamp(int min, int max, int value) {
            return Math.max(min, Math.min(max, value));
        }
    }
}
