package eu.jonahbauer.survey.persistence.entity;

import jakarta.persistence.*;
import jakarta.validation.constraints.Min;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Data;
import lombok.EqualsAndHashCode;
import org.hibernate.Hibernate;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.hibernate.annotations.SortNatural;

import java.util.Set;

@Entity
@Data
@Table(name = "question", uniqueConstraints = {
        @UniqueConstraint(name = "uq_question_survey_number", columnNames = {"survey", "number"})
})
@EqualsAndHashCode(of = "id")
public class Question implements Comparable<Question> {
    @Id
    @Column(name = "id")
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    /**
     * The question text.
     */
    @Column(name = "text", nullable = false)
    @NotBlank
    private String text;

    /**
     * The consecutive number of this question as part of a survey.
     */
    @Column(name = "number", nullable = false)
    @NotNull @Min(0)
    private Integer number;

    /**
     * The minimum number of answers that have to be selected.
     */
    @Column(name = "min_answers", nullable = false)
    @NotNull @Min(0)
    private Integer minAnswers;

    /**
     * The maximum number of answers that can be selected.
     */
    @Column(name = "max_answers", nullable = false)
    @NotNull @Min(1)
    private Integer maxAnswers;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "survey", nullable = false, foreignKey = @ForeignKey(name = "fk_question_survey"))
    @OnDelete(action = OnDeleteAction.CASCADE)
    @NotNull
    private Survey survey;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "question")
    @SortNatural
    private Set<Answer> answers;

    @Override
    public int compareTo(Question o) {
        return this.getNumber().compareTo(o.getNumber());
    }

    public boolean isRadio() {
        return minAnswers == 1 && maxAnswers == 1;
    }

    public String toString() {
        return "Question(id=" + this.getId() +
                ", text=" + this.getText() +
                ", number=" + this.getNumber() +
                ", minAnswers=" + this.getMinAnswers() +
                ", maxAnswers=" + this.getMaxAnswers() +
                (Hibernate.isInitialized(getAnswers()) ? ", answers=" + this.getAnswers() : "") +
                ")";
    }
}
