package eu.jonahbauer.survey.persistence.entity;

import jakarta.persistence.*;
import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.NoArgsConstructor;
import org.hibernate.Hibernate;
import org.hibernate.annotations.OnDelete;
import org.hibernate.annotations.OnDeleteAction;
import org.hibernate.annotations.SortNatural;

import java.time.Instant;
import java.util.Set;

/**
 * A survey is a collection of multiple {@linkplain Question questions} each having multiple possible
 * {@linkplain Answer answers}. A {@linkplain Submission submission} is a set of selected answers.
 */
@Entity
@Data
@Table(name = "survey")
@EqualsAndHashCode(of = "id")
@NoArgsConstructor
public class Survey {
    @Id
    @Column(name = "id", nullable = false)
    @GeneratedValue(strategy = GenerationType.IDENTITY)
    private Integer id;

    /**
     * The survey title.
     */
    @Column(name = "title", nullable = false)
    @NotBlank
    private String title;

    /**
     * A short, descriptive summary of the survey.
     */
    @Column(name = "summary", nullable = false)
    @NotBlank
    private String summary;

    @ManyToOne(fetch = FetchType.LAZY, optional = false)
    @JoinColumn(name = "owner", nullable = false, foreignKey = @ForeignKey(name = "fk_survey_owner"))
    @OnDelete(action = OnDeleteAction.CASCADE)
    @NotNull
    private User owner;

    @Column(name = "created", insertable = false, updatable = false, nullable = false)
    private Instant created;

    @OneToMany(cascade = CascadeType.ALL, mappedBy = "survey", fetch = FetchType.LAZY)
    @SortNatural
    private Set<Question> questions;

    @OneToMany(mappedBy = "survey", fetch = FetchType.LAZY)
    private Set<Submission> submissions;

    @Transient
    private transient Long submissionCount;

    public Survey(Integer id, String title, String summary, Instant created, long submissionCount) {
        this.id = id;
        this.title = title;
        this.summary = summary;
        this.created = created;
        this.submissionCount = submissionCount;
    }

    public String toString() {
        return "Survey(id=" + this.getId() +
                ", title=" + this.getTitle() +
                ", summary=" + this.getSummary() +
                ", created=" + this.getCreated() +
                (Hibernate.isInitialized(getQuestions()) ? ", questions=" + this.getQuestions() : "") +
                (Hibernate.isInitialized(getSubmissions()) ? ", submissions=" + this.getSubmissions() : "") +
                ")";
    }
}
